@@ -2,22 +2,23 @@ import sys
 import logging
 import json
 import subprocess, os
+from pathlib import Path
 from pprint import pprint
 
 import click
 
+
 from .dbc_reader import read_dbc, init
 from .dbc_writer import write_dbc
 from .c_generator import c_gen
-from .gui import gui
-from .validator import validate
+from .validator import validate, format_error
 from .spec import Spec
 from .docs import generate_docs
 from .version import VERSION
 
 
 def setup_logging() -> logging.Logger:
-    """ Setup Logger.
+    """Setup Logger.
 
     :return: logger object.
     """
@@ -36,7 +37,24 @@ def setup_logging() -> logging.Logger:
     return logger
 
 
-def get_spec(json_file: str, logger: logging.Logger) -> Spec:
+def report_validate(failed, force):
+    error = 0
+    if len(failed):
+        for level, msg in failed:
+            print(format_error(level, msg))
+
+            if level == "error":
+                error += 1
+
+        if error > 0:
+            if not force:
+                print("Too many error won't continue")
+                return True
+
+    return False
+
+
+def get_spec(json_file: str, force: bool = False) -> Spec:
     """Create Spec from json file path.
     :param json_file: path to the json file.
     :param logger: logger.
@@ -47,11 +65,9 @@ def get_spec(json_file: str, logger: logging.Logger) -> Spec:
     with open(json_file) as f:
         j = json.loads(f.read())
 
-
     spec.decompile(j)
-    failed = validate(logger, spec)
-    if len(failed):
-        print(msg)
+    failed = validate(spec)
+    if report_validate(failed, force):
         exit()
 
     return spec
@@ -62,7 +78,7 @@ def get_spec(json_file: str, logger: logging.Logger) -> Spec:
 @click.argument("json_file")
 @click.argument("device_config")
 def read_dbc_cmd(dbc: str, json_file: str, device_config: str):
-    """ Transform a DBC file into FCP json.
+    """Transform a DBC file into FCP json.
     :param dbc: dbc file path.
     :param json_file: FCP json file path.
     :param device_config: json with device name configuration.
@@ -71,22 +87,22 @@ def read_dbc_cmd(dbc: str, json_file: str, device_config: str):
 
     ```json
     {
-	"0": "amk_0",
-	"1": "amk_1",
-	"2": "amk_2",
-	"3": "amk_3",
-	"8": "dcu",
-	"9": "te",
-	"10": "dash",
-	"13": "arm",
-	"14": "bms_master",
-	"15": "bms_verbose",
-	"16": "iib",
-	"20": "ahrsf",
-	"21": "ahrsr",
-	"22": "gpsf",
-	"23": "gpsr",
-	"29": "isa"
+        "0": "amk_0",
+        "1": "amk_1",
+        "2": "amk_2",
+        "3": "amk_3",
+        "8": "dcu",
+        "9": "te",
+        "10": "dash",
+        "13": "arm",
+        "14": "bms_master",
+        "15": "bms_verbose",
+        "16": "iib",
+        "20": "ahrsf",
+        "21": "ahrsr",
+        "22": "gpsf",
+        "23": "gpsr",
+        "29": "isa"
     }```
     """
     logger = setup_logging()
@@ -97,27 +113,12 @@ def read_dbc_cmd(dbc: str, json_file: str, device_config: str):
 @click.argument("json_file")
 @click.argument("dbc")
 def write_dbc_cmd(json_file: str, dbc: str):
-    """ Transform FCP json file into a DBC
+    """Transform FCP json file into a DBC
     :param json_file: FCP json file path.
     :param dbc: dbc file path.
     """
 
-    logger = setup_logging()
-    
-    with open(json_file) as f:
-        j = json.loads(f.read())
-
-    spec = Spec()
-    spec.decompile(j)
-
-
-    failed = validate(logger, spec)
-    pprint(failed)
-
-    if len(failed) > 0:
-        print("Failed to validate json, refusing to write dbc")
-        exit()
-
+    spec = get_spec(json_file)
     write_dbc(spec, dbc, logger)
 
 
@@ -127,16 +128,18 @@ def write_dbc_cmd(json_file: str, dbc: str):
 @click.argument("json_file")
 @click.argument("skel")
 @click.option("--noformat", is_flag=True, default=False)
-def c_gen_cmd(templates: str, output: str, json_file: str, skel: str, noformat: bool):
+@click.option("--force", is_flag=True, default=False)
+def c_gen_cmd(templates: str, output: str, json_file: str, skel: str, noformat: bool, force: bool):
     """Transform FCP json into a C library.
     :param templates: jinja template directory.
     :param output: output directory.
     :param json_file: FCP json file path.
     :param skel: C skeleton directory.
     """
-
     logger = setup_logging()
-    c_gen(templates, output, json_file, skel, logger)
+
+    spec = get_spec(json_file, force)
+    c_gen(spec, templates, output, skel, logger)
 
     if noformat == False:
         subprocess.run(
@@ -150,12 +153,12 @@ def c_gen_cmd(templates: str, output: str, json_file: str, skel: str, noformat:
 @click.command(name="init")
 @click.argument("json_file")
 def init_cmd(json_file: str):
-    """Create a basic FCP json file. 
+    """Create a basic FCP json file.
     :param json_file: FCP json file path.
     """
 
     logger = setup_logging()
-    #init(json_file, logger)
+    # init(json_file, logger)
 
 
 @click.command(name="validate")
@@ -167,15 +170,17 @@ def validate_cmd(json_file: str):
 
     logger = setup_logging()
     with open(json_file) as f:
-        j = f.read()
-    
+        j = json.loads(f.read())
+
     spec = Spec()
     spec.decompile(j)
-    failed = validate(logger, spec)
+    failed = validate(spec)
     if len(failed) == 0:
         print("✓ JSON validated")
     else:
-        print("❌" + msg)
+        print("❌")
+        for level, msg in failed:
+            print(format_error(level, msg))
 
 
 @click.command(name="gui")
@@ -184,14 +189,29 @@ def gui_cmd(file: str):
     """Launch FCP json editor GUI.
     :param file: Optional FCP json file path
     """
+
+    from .gui import gui
+
     logger = setup_logging()
-    gui(file, logger)
+    gui(file)
+
+
+@click.command(name="gui2")
+@click.argument("file", type=Path, required=False)
+def gui_cmd2(file: Path):
+    """Launch FCP json editor GUI.
+    :param file: Optional FCP json file path
+    """
+
+    from .gui2 import gui2
+
+    gui2(file)
+
 
 @click.command()
 @click.argument("json_file")
 def dump_dev_list(json_file):
-    logger = setup_logging()
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev in spec.devices.keys():
         print(dev)
@@ -201,9 +221,7 @@ def dump_dev_list(json_file):
 @click.argument("json_file")
 @click.argument("device", required=False)
 def dump_msg_list(json_file, device):
-    logger = setup_logging()
-
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev_name, dev in spec.devices.items():
         if device != None and device != dev_name:
@@ -217,9 +235,7 @@ def dump_msg_list(json_file, device):
 @click.argument("json_file")
 @click.argument("device", required=False)
 def dump_cfg_list(json_file, device):
-    logger = setup_logging()
-
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev_name, dev in spec.devices.items():
         if device != None and device != dev_name:
@@ -233,9 +249,7 @@ def dump_cfg_list(json_file, device):
 @click.argument("json_file")
 @click.argument("device", required=False)
 def dump_cmd_list(json_file, device):
-    logger = setup_logging()
-
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev_name, dev in spec.devices.items():
         if device != None and device != dev_name:
@@ -248,8 +262,7 @@ def dump_cmd_list(json_file, device):
 @click.command()
 @click.argument("json_file")
 def dump_log_list(json_file):
-    logger = setup_logging()
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for log in spec.logs.keys():
         print(log)
@@ -259,8 +272,7 @@ def dump_log_list(json_file):
 @click.argument("json_file")
 @click.argument("message", required=False)
 def dump_signal_list(json_file, message):
-    logger = setup_logging()
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev in spec.devices.values():
         for msg in dev.msgs.values():
@@ -269,12 +281,12 @@ def dump_signal_list(json_file, message):
             for signal in msg.signals.values():
                 print(signal)
 
+
 @click.command()
 @click.argument("json_file")
 @click.argument("message", required=False)
 def dump_signal_names(json_file, message):
-    logger = setup_logging()
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev in spec.devices.values():
         for msg in dev.msgs.values():
@@ -283,12 +295,12 @@ def dump_signal_names(json_file, message):
             for signal in msg.signals.values():
                 print(signal.name)
 
+
 @click.command()
 @click.argument("json_file")
 @click.argument("log")
 def print_log(json_file, log):
-    logger = setup_logging()
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     print(spec.get_log(log))
 
@@ -297,9 +309,7 @@ def print_log(json_file, log):
 @click.argument("json_file")
 @click.argument("dev")
 def print_dev(json_file, dev):
-    logger = setup_logging()
-
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     print(spec.get_device(dev))
 
@@ -308,8 +318,7 @@ def print_dev(json_file, dev):
 @click.argument("json_file")
 @click.argument("message")
 def print_msg(json_file, message):
-    logger = setup_logging()
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev in spec.devices.values():
         for name, msg in dev.msgs.items():
@@ -322,9 +331,7 @@ def print_msg(json_file, message):
 @click.argument("json_file")
 @click.argument("signal")
 def print_signal(json_file, signal):
-    logger = setup_logging()
-
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
 
     for dev in spec.devices.values():
         for msg in dev.msgs.values():
@@ -339,7 +346,7 @@ def print_signal(json_file, signal):
 @click.argument("out")
 @click.argument("link_location", required=False)
 def docs(json_file: str, out: str, link_location: str):
-    """ Generate FCP documentation. 
+    """Generate FCP documentation.
     :param json_file: FCP json file path.
     :param out: output directory.
     """
@@ -349,14 +356,29 @@ def docs(json_file: str, out: str, link_location: str):
 
     logger = setup_logging()
 
-    spec = get_spec(json_file, logger)
+    spec = get_spec(json_file)
     generate_docs(spec, out, link_location, logger)
 
+@click.command("fix")
+@click.argument("src")
+@click.argument("dst")
+def fix(src: str, dst: str):
+    print("Warning: This operation may lose some data")
+    spec = Spec()
+    with open(src) as f:
+        j = json.loads(f.read())
+
+    spec.decompile(j)
+    d = spec.compile()
+    with open(dst, "w") as f:
+        f.write(json.dumps(d, indent=4))
+
+
 @click.group(invoke_without_command=True)
 @click.option("--version", is_flag=True, default=False)
 def main(version):
-    """CLI utility for managment of FCP JSON files."""
-        
+    """CLI utility for managment of FCP JSON files. """
+
     if len(sys.argv) == 1:
         print("fcp cli util.\nVersion:", VERSION, "\nFor usage see fcp --help")
     if version:
@@ -370,21 +392,20 @@ main.add_command(c_gen_cmd)
 main.add_command(init_cmd)
 main.add_command(validate_cmd)
 main.add_command(gui_cmd)
-main.add_command(dump_dev_list)
-main.add_command(dump_msg_list)
-main.add_command(dump_cfg_list)
-main.add_command(dump_cmd_list)
-main.add_command(dump_log_list)
-main.add_command(dump_signal_list)
-main.add_command(dump_signal_names)
-main.add_command(print_log)
-main.add_command(print_dev)
-main.add_command(print_msg)
-main.add_command(print_signal)
+#main.add_command(gui_cmd2)
+#main.add_command(dump_dev_list)
+#main.add_command(dump_msg_list)
+#main.add_command(dump_cfg_list)
+#main.add_command(dump_cmd_list)
+#main.add_command(dump_log_list)
+#main.add_command(dump_signal_list)
+#main.add_command(dump_signal_names)
+#main.add_command(print_log)
+#main.add_command(print_dev)
+#main.add_command(print_msg)
+#main.add_command(print_signal)
 main.add_command(docs)
+main.add_command(fix)
 
 if __name__ == "__main__":
     main()
-
-
-
